"""deal summary lsoa-region offer improvment

Revision ID: 7ec59cb49f09
Revises: 3d8b428eb9d3
Create Date: 2019-01-25 13:26:30.376649

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = '7ec59cb49f09'
down_revision = '3d8b428eb9d3'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    sql = '''
    drop index if exists name_key_lookup_table;
    create unique index name_key_lookup_table on lookup_table(lookup_name, lookup_key); 

    CREATE OR REPLACE VIEW public.offer AS
    SELECT offer.value ->> 'id'::text AS offer_id,
       deal.collection,
       deal.id AS deal_table_id,
       deal.deal_id,
       deal.deal ->> 'title'::text AS deal_title,
       deal.deal ->> 'status'::text AS deal_status,
       offer.value AS offer
    FROM deal,
     LATERAL jsonb_array_elements(deal.deal -> 'offers'::text) offer(value)
           where offer.value->'id' is not null;

    drop materialized view collection_summary;
    drop materialized view deal_summary;

    CREATE MATERIALIZED VIEW public.deal_summary
      AS
    SELECT deal.id AS deal_table_id,
        deal.deal_id,
        deal.collection,
        (deal.deal -> 'recipientOrganization'::text) ->> 'name'::text AS recipient_organization_name,
        (deal.deal -> 'recipientOrganization'::text) ->> 'id'::text AS recipient_organization_id,
        (deal.deal -> 'arrangingOrganization'::text) ->> 'name'::text AS arraging_organization_name,
        (deal.deal -> 'arrangingOrganization'::text) ->> 'id'::text AS arraging_organization_id,
        deal.deal ->> 'status'::text AS status,
        deal.deal ->> 'currency'::text AS currency,
        COALESCE(convert_to_numeric(deal.deal ->> 'estimatedValue'::text)::numeric, 0::numeric) AS estimated_value,
        COALESCE(convert_to_numeric(deal.deal ->> 'value'::text)::numeric, 0::numeric) AS value,
        COALESCE(os.offer_count, 0::bigint) AS offer_count,
        COALESCE(os.csu_standard_mark_awarded, 0::bigint) AS csu_standard_mark_awarded,
        COALESCE(os.investment_target, 0::bigint) AS investment_target,
        COALESCE(os.minimum_investment_target, 0::bigint) AS minimum_investment_target,
        COALESCE(os.maximum_investment_target, 0::bigint) AS maximum_investment_target,
        COALESCE(ps.project_estimated_value, 0::bigint) AS project_estimated_value,
        COALESCE(ps.project_raised_value, 0::bigint) AS project_raised_value,
        COALESCE(ps.project_count, 0::bigint) AS project_count,
        COALESCE(ps.asset_purchase_price, 0::numeric) AS asset_purchase_price,
        COALESCE(ps.asset_quantity, 0::numeric) AS asset_quantity,
        COALESCE(ps.asset_count, 0::numeric) AS asset_count,
        COALESCE(ps.asset_value, 0::numeric) AS asset_value,
        COALESCE(gs.grant_count, 0::bigint) AS grant_count,
        COALESCE(gs.is_match_funding, 0::bigint) AS is_match_funding,
        COALESCE(gs.grant_amount_committed, 0::bigint) AS grant_amount_committed,
        COALESCE(gs.grant_amount_disbursed, 0::bigint) AS grant_amount_disbursed,
        COALESCE(gs.grant_amount_requested, 0::bigint) AS grant_amount_requested,
        COALESCE(es.equity_count, 0::bigint) AS equity_count,
        COALESCE(es.equity_value, 0::bigint) AS equity_value,
        COALESCE(es.equity_estimated_value, 0::bigint) AS equity_estimated_value,
        COALESCE(cs.credit_count, 0::bigint) AS credit_count,
        COALESCE(cs.credit_estimated_value, 0::numeric) AS credit_estimated_value,
        COALESCE(cs.credit_value, 0::numeric) AS credit_value,
        loc->>'geoCode' lsoa, 
        lt1.lookup_name imd_type, lt1.data imd_data,
        case when upper(left(deal->'recipientOrganization'->>'postalCode', 2)) = 'BT' then 'Northern Ireland' else lt3.data ->> 'nuts1_name' end nuts1,
        lt3.data ->> 'nuts2_name' nuts2,
        deal.deal
       FROM deal
         LEFT JOIN offer_summary os ON os.deal_table_id = deal.id
         LEFT JOIN project_summary ps ON ps.deal_table_id = deal.id
         LEFT JOIN grant_summary gs ON gs.deal_table_id = deal.id
         LEFT JOIN equity_summary es ON es.deal_table_id = deal.id
         LEFT JOIN credit_summary cs ON cs.deal_table_id = deal.id
             left join jsonb_array_elements(deal->'recipientOrganization'->'location') as loc on loc->>'geoCodeType' = 'lsoa' 
             LEFT JOIN lookup_table lt1 on lt1.lookup_name in ('imd_england', 'imd_scotland', 'imd_wales') and lt1.lookup_key = loc->>'geoCode'
             LEFT JOIN lookup_table lt2 on lt2.lookup_name = 'nuts3_post' and lt2.lookup_key = upper(replace(replace(trim(deal->'recipientOrganization'->>'postalCode'), ' ',''), E'\n', ''))
             LEFT JOIN lookup_table lt3 on lt3.lookup_name = 'nuts2_codes' and lt3.lookup_key = lt2.data ->> 'NUTS_2';


                                                                                                                                                                                                             
    CREATE MATERIALIZED VIEW public.collection_summary
    AS
     SELECT deal_summary.collection,
        count(*) AS deal_count,
        sum(deal_summary.estimated_value) AS estimated_value,
        sum(deal_summary.value) AS value,
        sum(deal_summary.offer_count) AS offer_count,
        sum(deal_summary.csu_standard_mark_awarded) AS csu_standard_mark_awarded,
        sum(deal_summary.investment_target) AS investment_target,
        sum(deal_summary.minimum_investment_target) AS minimum_investment_target,
        sum(deal_summary.maximum_investment_target) AS maximum_investment_target,
        sum(deal_summary.project_estimated_value) AS project_estimated_value,
        sum(deal_summary.project_raised_value) AS project_raised_value,
        sum(deal_summary.project_count) AS project_count,
        sum(deal_summary.asset_purchase_price) AS asset_purchase_price,
        sum(deal_summary.asset_quantity) AS asset_quantity,
        sum(deal_summary.asset_count) AS asset_count,
        sum(deal_summary.asset_value) AS asset_value,
        sum(deal_summary.grant_count) AS grant_count,
        sum(deal_summary.is_match_funding) AS is_match_funding,
        sum(deal_summary.grant_amount_committed) AS grant_amount_committed,
        sum(deal_summary.grant_amount_disbursed) AS grant_amount_disbursed,
        sum(deal_summary.grant_amount_requested) AS grant_amount_requested,
        sum(deal_summary.equity_count) AS equity_count,
        sum(deal_summary.equity_value) AS equity_value,
        sum(deal_summary.equity_estimated_value) AS equity_estimated_value,
        sum(deal_summary.credit_count) AS credit_count,
        sum(deal_summary.credit_estimated_value) AS credit_estimated_value,
        sum(deal_summary.credit_value) AS credit_value
       FROM deal_summary
      GROUP BY deal_summary.collection;
    '''
    op.execute(sql)
    
    # ### end Alembic commands ###


def downgrade():
    pass
    # ### commands auto generated by Alembic - please adjust! ###
    # ### end Alembic commands ###



"""create materialized views

Revision ID: 3d8b428eb9d3
Revises: f0771214ae1e
Create Date: 2019-01-22 10:52:50.589667

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = '3d8b428eb9d3'
down_revision = 'f0771214ae1e'
branch_labels = None
depends_on = None


def upgrade():
    create_view_statements = '''
    drop view collection_summary;
    drop view deal_summary;

    create MATERIALIZED view deal_summary
    as
    select 
       id deal_table_id,
       deal_id,
       collection,
       deal->'recipientOrganization'->>'name' as recipient_organization_name,
       deal->'recipientOrganization'->>'id' as recipient_organization_id,
       deal->'arrangingOrganization'->>'name' as arraging_organization_name,
       deal->'arrangingOrganization'->>'id' as arraging_organization_id,
       deal->>'status' as status,
       deal->>'currency' currency,
       coalesce(convert_to_numeric(deal->>'estimatedValue')::numeric, 0) as estimated_value,
       coalesce(convert_to_numeric(deal->>'value')::numeric, 0) as value,
       coalesce(offer_count, 0) as offer_count,
       coalesce(csu_standard_mark_awarded, 0) as csu_standard_mark_awarded,
       coalesce(investment_target, 0) as investment_target,
       coalesce(minimum_investment_target, 0) as minimum_investment_target,
       coalesce(maximum_investment_target, 0) as maximum_investment_target,
       coalesce(project_estimated_value, 0) as project_estimated_value,
       coalesce(project_raised_value, 0) as project_raised_value,
       coalesce(project_count, 0) as project_count,
       coalesce(asset_purchase_price, 0) as asset_purchase_price,
       coalesce(asset_quantity, 0) as asset_quantity,
       coalesce(asset_count, 0) as asset_count,
       coalesce(asset_value, 0) as asset_value,
       coalesce(grant_count, 0) as grant_count,
       coalesce(is_match_funding, 0) as is_match_funding,
       coalesce(grant_amount_committed, 0) as grant_amount_committed,
       coalesce(grant_amount_disbursed, 0) as grant_amount_disbursed,
       coalesce(grant_amount_requested, 0) as grant_amount_requested,
       coalesce(equity_count, 0) as equity_count,
       coalesce(equity_value, 0) as equity_value,
       coalesce(equity_estimated_value, 0) as equity_estimated_value,
       coalesce(credit_count, 0) as credit_count,
       coalesce(credit_estimated_value, 0) as credit_estimated_value,
       coalesce(credit_value, 0) as credit_value,
       deal
    from deal
       left join offer_summary os on os.deal_table_id = deal.id
       left join project_summary ps on ps.deal_table_id = deal.id
       left join grant_summary gs on gs.deal_table_id = deal.id
       left join equity_summary es on es.deal_table_id = deal.id
       left join credit_summary cs on cs.deal_table_id = deal.id;


    create MATERIALIZED view collection_summary
    as
    select 
       collection,
       count(*) deal_count,
       sum(estimated_value) as estimated_value,
       sum(value) as value,
       sum(offer_count) as offer_count,
       sum(csu_standard_mark_awarded) as csu_standard_mark_awarded,
       sum(investment_target) as investment_target,
       sum(minimum_investment_target) as minimum_investment_target,
       sum(maximum_investment_target) as maximum_investment_target,
       sum(project_estimated_value) as project_estimated_value,
       sum(project_raised_value) as project_raised_value,
       sum(project_count) as project_count,
       sum(asset_purchase_price) as asset_purchase_price,
       sum(asset_quantity) as asset_quantity,
       sum(asset_count) as asset_count,
       sum(asset_value) as asset_value,
       sum(grant_count) as grant_count,
       sum(is_match_funding) as is_match_funding,
       sum(grant_amount_committed) as grant_amount_committed,
       sum(grant_amount_disbursed) as grant_amount_disbursed,
       sum(grant_amount_requested) as grant_amount_requested,
       sum(equity_count) as equity_count,
       sum(equity_value) as equity_value,
       sum(equity_estimated_value) as equity_estimated_value,
       sum(credit_count) as credit_count,
       sum(credit_estimated_value) as  credit_estimated_value,
       sum(credit_value) as credit_value
    from deal_summary
    group by 1;
    '''

    op.execute(create_view_statements)

def downgrade():
    pass



"""make lookup table

Revision ID: f0771214ae1e
Revises: 3c5d39510935
Create Date: 2019-01-10 22:09:52.192410

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = 'f0771214ae1e'
down_revision = '3c5d39510935'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('lookup_table',
    sa.Column('id', sa.Integer(), nullable=False),
    sa.Column('lookup_name', sa.Text(), nullable=False),
    sa.Column('lookup_key', sa.Text(), nullable=True),
    sa.Column('data', postgresql.JSONB(astext_type=sa.Text()), nullable=True),
    sa.PrimaryKeyConstraint('id')
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('lookup_table')
    # ### end Alembic commands ###



"""Fix bad casting

Revision ID: c2f1fafd2225
Revises: bff84c33d64d
Create Date: 2019-01-10 15:41:34.358222

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = 'c2f1fafd2225'
down_revision = 'bff84c33d64d'
branch_labels = None
depends_on = None


def upgrade():
    create_view_statements = '''
    CREATE EXTENSION if not exists  plpgsql;

    CREATE OR REPLACE FUNCTION convert_to_numeric(v_input text)
        RETURNS INTEGER AS $$
    DECLARE v_num_value NUMERIC DEFAULT NULL;
    BEGIN
        BEGIN
            v_num_value := v_input::NUMERIC;
        EXCEPTION WHEN OTHERS THEN
            RETURN NULL;
        END;
    RETURN v_num_value;
    END;
    $$ LANGUAGE plpgsql;


    drop view collection_summary;
    drop view deal_summary;

    drop view offer_summary;
    create view offer_summary
    as
    select 
      deal_table_id, 
      count(*) offer_count,
      sum(case when offer.offer->'csuStandardMark' is not null AND (offer.offer->'csuStandardMark'->>'awarded')::bool then 1 else 0 end) csu_standard_mark_awarded,
      sum(coalesce(convert_to_numeric(offer.offer->>'investmentTarget'), 0)) investment_target,
      sum(coalesce(convert_to_numeric(offer.offer->>'minimumInvestmentTarget'), 0)) minimum_investment_target,
      sum(coalesce(convert_to_numeric(offer.offer->>'maximumInvestmentTarget'), 0)) maximum_investment_target
    from offer group by 1;

    drop view project_summary;
    create view project_summary
    as
    select 
        project.deal_table_id,
        count(*) as project_count,
        sum(coalesce(convert_to_numeric(project->>'estimatedValue'), 0)) project_estimated_value,
        sum(coalesce(convert_to_numeric(project->>'raisedValue'), 0)) project_raised_value,
        sum(asset_count) asset_count,
        sum(asset_value) asset_value,
        sum(asset_purchase_price) asset_purchase_price,
        sum(asset_quantity) asset_quantity
    from project 
    left join 
       (select 
          deal_table_id, 
          count(*) as asset_count, 
          sum(case when jsonb_typeof(asset.value->'totalValue') = 'number' then (asset.value->>'totalValue')::numeric else 0 end) asset_value,
          sum(case when jsonb_typeof(asset.value->'purcahasePrice') = 'number' then (asset.value->>'purcahasePrice')::numeric else 0 end) asset_purchase_price,
          sum(case when jsonb_typeof(asset.value->'quantity') = 'number' then (asset.value->>'quantity')::numeric else 0 end) asset_quantity
       from 
          project, 
          jsonb_array_elements(project->'assets') as asset group by 1
       ) assets 
       on assets.deal_table_id = project.deal_table_id 
    group by 1;


    drop view grant_summary;
    create view grant_summary
    as
    select 
      deal_table_id, 
      count(*) grant_count,
      sum(case when ("grant"->>'isMatchFunding')::bool then 1 else 0 end) is_match_funding,
      sum(coalesce(convert_to_numeric("grant"."grant"->>'amountRequested'), 0)) grant_amount_requested,
      sum(coalesce(convert_to_numeric("grant"."grant"->>'amountCommitted'), 0)) grant_amount_committed,
      sum(coalesce(convert_to_numeric("grant"."grant"->>'amountDisbursed'), 0)) grant_amount_disbursed
    from "grant" group by 1;


    drop view equity_summary;
    create view equity_summary
    as
    select 
      deal_table_id, 
      count(*) equity_count,
      sum(coalesce(convert_to_numeric("equity"."equity"->>'value'), 0)) equity_value,
      sum(coalesce(convert_to_numeric("equity"."equity"->>'estimatedValue'), 0)) equity_estimated_value
    from "equity" group by 1;


    create view deal_summary
    as
    select 
       id deal_table_id,
       deal_id,
       collection,
       deal->'recipientOrganization'->>'name' as recipient_organization_name,
       deal->'recipientOrganization'->>'id' as recipient_organization_id,
       deal->'arrangingOrganization'->>'name' as arraging_organization_name,
       deal->'arrangingOrganization'->>'id' as arraging_organization_id,
       deal->>'status' as status,
       deal->>'currency' currency,
       coalesce(convert_to_numeric(deal->>'estimatedValue')::numeric, 0) as estimated_value,
       coalesce(convert_to_numeric(deal->>'value')::numeric, 0) as value,
       coalesce(offer_count, 0) as offer_count,
       coalesce(csu_standard_mark_awarded, 0) as csu_standard_mark_awarded,
       coalesce(investment_target, 0) as investment_target,
       coalesce(minimum_investment_target, 0) as minimum_investment_target,
       coalesce(maximum_investment_target, 0) as maximum_investment_target,
       coalesce(project_estimated_value, 0) as project_estimated_value,
       coalesce(project_raised_value, 0) as project_raised_value,
       coalesce(project_count, 0) as project_count,
       coalesce(asset_purchase_price, 0) as asset_purchase_price,
       coalesce(asset_quantity, 0) as asset_quantity,
       coalesce(asset_count, 0) as asset_count,
       coalesce(asset_value, 0) as asset_value,
       coalesce(grant_count, 0) as grant_count,
       coalesce(is_match_funding, 0) as is_match_funding,
       coalesce(grant_amount_committed, 0) as grant_amount_committed,
       coalesce(grant_amount_disbursed, 0) as grant_amount_disbursed,
       coalesce(grant_amount_requested, 0) as grant_amount_requested,
       coalesce(equity_count, 0) as equity_count,
       coalesce(equity_value, 0) as equity_value,
       coalesce(equity_estimated_value, 0) as equity_estimated_value,
       coalesce(credit_count, 0) as credit_count,
       coalesce(credit_estimated_value, 0) as credit_estimated_value,
       deal
    from deal
       left join offer_summary os on os.deal_table_id = deal.id
       left join project_summary ps on ps.deal_table_id = deal.id
       left join grant_summary gs on gs.deal_table_id = deal.id
       left join equity_summary es on es.deal_table_id = deal.id
       left join credit_summary cs on cs.deal_table_id = deal.id;


    create view collection_summary
    as
    select 
       collection,
       count(*) deal_count,
       sum(estimated_value) as estimated_value,
       sum(value) as value,
       sum(offer_count) as offer_count,
       sum(csu_standard_mark_awarded) as csu_standard_mark_awarded,
       sum(investment_target) as investment_target,
       sum(minimum_investment_target) as minimum_investment_target,
       sum(maximum_investment_target) as maximum_investment_target,
       sum(project_estimated_value) as project_estimated_value,
       sum(project_raised_value) as project_raised_value,
       sum(project_count) as project_count,
       sum(asset_purchase_price) as asset_purchase_price,
       sum(asset_quantity) as asset_quantity,
       sum(asset_count) as asset_count,
       sum(asset_value) as asset_value,
       sum(grant_count) as grant_count,
       sum(is_match_funding) as is_match_funding,
       sum(grant_amount_committed) as grant_amount_committed,
       sum(grant_amount_disbursed) as grant_amount_disbursed,
       sum(grant_amount_requested) as grant_amount_requested,
       sum(equity_count) as equity_count,
       sum(equity_value) as equity_value,
       sum(equity_estimated_value) as equity_estimated_value,
       sum(credit_count) as credit_count,
       sum(credit_estimated_value) as  credit_estimated_value
    from deal_summary
    group by 1;
    '''
    op.execute(create_view_statements)

    # ### end Alembic commands ###


def downgrade():
    pass



import datetime
import html
import json
import os
import getpass
import csv
from collections import OrderedDict
import io

import alembic.config
import jinja2
from flattentool import unflatten
import gspread
from gspread.utils import fill_gaps
from openpyxl import Workbook

from sedldata.database import Database


def in_notebook():
    if 'JPY_PARENT_PID' in os.environ:
        return True
    return False


def xl_to_json(infile, outfile):
    try:
        unflatten(
                input_name=infile,
                output_name=outfile,
                input_format='xlsx',
                metatab_name='Meta',
                metatab_vertical_orientation=True,
                root_list_path='deals',
                id_name='id',
                cell_source_map='sourcemap-' + outfile,
                root_id='')
        with open(outfile,'r') as json_file:
            data = json.load(json_file)
        with open('sourcemap-' + outfile,'r') as source_map:
            source_map_data = json.load(source_map)
    except Exception as e:
        raise e

    return data, source_map_data

table = jinja2.Template(
'''
<table class="dataframe">
    <thead>
    <tr>
      {% for header in headers %}
        <th style="text-align: left; vertical-align: top">{{ header }}</th>
      {% endfor %}
    </tr>
    </thead>
    <tbody>
      {% for row in data %}
        <tr>
          {% for cell in row %}
              <td style="text-align: left; vertical-align: top">
                {% if not display_full_json %}
                    <pre>{{ cell|truncate(50) }}</pre>
                {% else %}
                    <pre>{{ cell }}</pre>
                {% endif %}
              </td>
          {% endfor %}
        </tr>
      {% endfor %}
    </tbody>
</table>
'''
)


def generate_rows(result, limit, to_html=False):
    for num, row in enumerate(result):
        if num == limit:
            break
        if to_html:
            yield [json.dumps(item, indent=2) if isinstance(item, dict) else html.escape(str(item)) for item in row]
        else:
            yield [item for item in row]


class Session:

    def __init__(self, db_uri=None):
        if in_notebook():
            if not db_uri:
                db_uri = 'postgresql://sedldata:{password}@46.43.2.250:5432/sedldata'
            if '{password}' in db_uri:
                db_uri = db_uri.format(password=getpass.getpass("Enter database password:  "))
            self.db = Database(db_uri)
            self.db.upgrade()
        else:
            self.db = Database()

        self.gspread_client = None

    def get_gspread_client(self):
        if not self.gspread_client:
            from google.colab import auth
            from oauth2client.client import GoogleCredentials
            auth.authenticate_user()
            self.gspread_client = gspread.authorize(GoogleCredentials.get_application_default())
        return self.gspread_client

    def load_xlsx(self, collection=None, infile=None, outfile='output.json', append=False):
        if not collection and in_notebook():
            collection = input('Please state collections name: ')
        if not collection:
            raise ValueError('You need to input a non-empty collection name!')

        if in_notebook() and not infile:
            from google.colab import files
            print('Upload your xlsx SEDL file:')
            uploaded = files.upload()
            for file_name in uploaded:
                infile = 'uploaded.xlsx'
                with open(infile, '+wb') as f:
                    f.write(uploaded[file_name])
                break
        
        if not infile:
            raise ValueError('You need to state an input file')

        unflattened, source_map = xl_to_json(infile, outfile)
        deal_indexes = set()
        org_indexes = set()

        for path, value in source_map.items():
            split_path = path.split('/')
            if len(split_path) < 2:
                continue
            index = int(split_path[1])
            for source_item in value:
                if source_item[0].lower().strip().startswith('deal'):
                    deal_indexes.add(index)
                if source_item[0].lower().strip().startswith('org'):
                    org_indexes.add(index)

        metadata = {key: value for key, value in unflattened.items() if key != 'deals'}

        deals = []
        orgs = []
        for num, obj in enumerate(unflattened['deals']):
            now = datetime.datetime.now()
            obj_id = obj.get('id')
            if num in deal_indexes:
                if not obj_id:
                    print('WARNING: object {} has no id field'.format(obj))
                    continue
                obj_id = obj.get('id')
                deals.append(dict(date_loaded=now, collection=collection, deal=obj, deal_id=obj_id, metadata=metadata))
            if num in org_indexes:
                if not obj_id:
                    obj['id'] = str(num)
                    print('WARNING: object {} has no id field'.format(obj))
                obj_id = obj.get('id')
                orgs.append(dict(date_loaded=now, collection=collection, organization=obj, org_id=obj_id, metadata=metadata))

        
        with self.db.engine.begin() as connection:
            if not append:
                connection.execute(''' delete from deal where collection = %s ''', collection)
                connection.execute(''' delete from organization where collection = %s ''', collection)

            if deals:
                insert = self.db.deal_table.insert()
                connection.execute(insert, deals)
            if orgs:
                insert = self.db.org_table.insert()
                connection.execute(insert, orgs)

        now = datetime.datetime.now()
        self.refresh_views()
        print("Loaded %s at: %s" % (collection, now))


    def refresh_views(self):
        with self.db.engine.begin() as connection:
            connection.execute('''REFRESH MATERIALIZED VIEW deal_summary; REFRESH MATERIALIZED VIEW collection_summary;''')


    def load_google_sheet(self, sheet_url=None, collection=None):
        if not in_notebook:
            raise Exception("Con not use google spreadsheets outside colab notebooks")

        xlsx_workbook = Workbook(write_only=True)
        gc = self.get_gspread_client()
        spreadsheet = gc.open_by_url(sheet_url)

        worksheets = spreadsheet.worksheets()

        for sheet in worksheets:
            if sheet.title.startswith('#'):
                continue
            xlsx_sheet = xlsx_workbook.create_sheet(sheet.title)
            data = spreadsheet.values_get(sheet.title, params={"valueRenderOption":"UNFORMATTED_VALUE", "dateTimeRenderOption": "FORMATTED_STRING"})
            values = data.get('values')
            if not values:
                continue
            data = fill_gaps(values)

            for row in data:
                xlsx_sheet.append(row)

        xlsx_workbook.save('converted.xlsx')
        self.load_xlsx(collection=collection, infile='converted.xlsx')


    def delete_collection(self, collection):
        with self.db.engine.begin() as connection:
            connection.execute('''delete from deal where collection = %s ''', collection)
            connection.execute('''delete from organization where collection = %s ''', collection)
        self.refresh_views()


    def get_results(self, sql, limit=-1, params=None, html=False, as_dicts=True):

        with self.db.engine.begin() as connection:
            params = params or []
            sql_result = connection.execute(sql, *params)
            if sql_result.returns_rows:
                results = {"headers": sql_result.keys()}
                if as_dicts:
                    data = []
                    for row in generate_rows(sql_result, limit, html):
                        data.append(OrderedDict(zip(sql_result.keys(), row)))
                    results["data"] = data
                else:
                    results["data"] = [row for row in generate_rows(sql_result, limit, html)]

                return results
            else:
                return "Success"


    def run_sql(self, sql, limit=100, params=None, display_full_json=False):
        from IPython.core.display import display, HTML
        results = self.get_results(sql, limit, params, html=True, as_dicts=False)
        if results == 'Success':
            return results
        results['display_full_json'] = display_full_json
        display(HTML(table.render(results)))


    def add_lookup_from_csv(self, lookup_name, key_name, infile=None):

        if in_notebook() and not infile:
            from google.colab import files
            print('Upload your csv file:')
            uploaded = files.upload()
            for file_name in uploaded:
                infile = 'uploaded.csv'
                with open(infile, '+wb') as f:
                    f.write(uploaded[file_name])
                break

        if not infile:
            raise ValueError('You need to state an input file')


        with self.db.engine.begin() as connection:
            connection.execute('''delete from lookup_table where lookup_name = %s''', lookup_name)
            rows = []
            with io.open(infile, "r", encoding='utf-8-sig') as f:
                reader = csv.DictReader(f)
                for row in reader:
                    rows.append(dict(lookup_name=lookup_name, data=row, lookup_key=row.get(key_name)))

            insert = self.db.lookup_table.insert()
            insert.execute(rows)

    def add_to_dashboard(self, collection):
        with self.db.engine.begin() as connection:
            connection.execute('''delete from dashboard where collection = %s ''', collection)
            connection.execute('''insert into dashboard values (%s) ''', collection)

    def delete_from_dashboard(self, collection):
        with self.db.engine.begin() as connection:
            connection.execute('''delete from dashboard where collection = %s ''', collection)



"""Add dashbord table

Revision ID: d9fb04ce0ad8
Revises: 7ec59cb49f09
Create Date: 2019-01-28 22:19:28.833566

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = 'd9fb04ce0ad8'
down_revision = '7ec59cb49f09'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('dashboard',
    sa.Column('collection', sa.Text(), nullable=False),
    sa.UniqueConstraint('collection')
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('dashboard')
    # ### end Alembic commands ###
